"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = exports.eslintExtensionAll = void 0;

var _set = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/set"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _path = require("path");

var _exec = _interopRequireDefault(require("../../cmd/exec"));

var _helper = require("../../helper");

const configs = ['.eslintrc.js', '.eslintrc.cjs', '.eslintrc.yaml', '.eslintrc.yml', '.eslintrc.json', '.eslintrc'];
const ignoreFiles = new _set.default(['babel.config.js', 'jest.config.js', 'webpack.config.js']);
const eslintExtensionAll = 'js,ts,tsx';
exports.eslintExtensionAll = eslintExtensionAll;

const eslint = async ({
  cwd,
  cfg,
  log
}, {
  write,
  files
}) => {
  const hasConfigFile = await (0, _helper.existConfigFile)(cwd, configs);
  const cmd = './node_modules/.bin/eslint';
  const args = ['--color', '--max-warnings', '0'];

  if (!hasConfigFile) {
    args.push('--config', (0, _path.relative)(cwd, (0, _path.join)(cfg, 'eslintrc.json')));
  }

  if (write) {
    args.push('--fix');
  } else {
    args.push('--format', 'codeframe');
  }

  if (files) {
    const filteredFiles = (0, _filter.default)(files).call(files, file => !ignoreFiles.has((0, _path.basename)(file)));

    if (filteredFiles.length === 0) {
      log.info('No files found for eslint');
      return;
    }

    args.push(...filteredFiles);
  } else {
    args.push('--ext', `"${eslintExtensionAll}"`);
    args.push('.');
  }

  log.info('Run eslint');
  await (0, _exec.default)({
    cmd,
    args,
    cwd,
    log
  });
};

var _default = eslint;
exports.default = _default;