"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/includes"));

var _slice = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/slice"));

var _sort = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/sort"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _startsWith = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/starts-with"));

var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/trim"));

var _promises = require("fs/promises");

var _path = require("path");

var _toolLib = require("@kettil/tool-lib");

var _licenseHeuristics = _interopRequireDefault(require("../../../../configs/licenseHeuristics"));

var _access = _interopRequireDefault(require("../../../cmd/access"));

var _exec = _interopRequireDefault(require("../../../cmd/exec"));

var _readFile = _interopRequireDefault(require("../../../cmd/readFile"));

/* eslint-disable no-restricted-syntax */

/* eslint-disable no-await-in-loop */
const regexpFilename = /^(copying|license|license-\w+|licence|licence-\w+|readme)(\.markdown|\.md|\.txt)?$/iu;

const getPackageInfo = async ({
  cwd,
  log
}, {
  packagePath,
  ignorePackages,
  licensePackages
}) => {
  var _context, _context2;

  const packageJsonPath = (0, _path.join)(packagePath, 'package.json');
  const hasPackageJson = await (0, _access.default)(packageJsonPath);

  if (!hasPackageJson) {
    // No node.js package
    return undefined;
  }

  const {
    name,
    license,
    version
  } = await (0, _readFile.default)(packageJsonPath, true);

  if (typeof name !== 'string') {
    throw new TypeError(`Package name is not defined (${packageJsonPath})`);
  }

  if ((0, _includes.default)(ignorePackages).call(ignorePackages, name)) {
    return undefined;
  }

  const data = {
    path: (0, _slice.default)(packagePath).call(packagePath, Math.max(0, cwd.length + 1)),
    version: typeof version === 'string' ? version : 'UNKNOWN',
    license: 'UNKNOWN',
    name
  }; // Manual license assignment

  const licensePackage = licensePackages[data.name];

  if ((0, _toolLib.isObject)(licensePackage) && typeof licensePackage[data.version] === 'string') {
    return { ...data,
      license: licensePackage[data.version]
    };
  } // License from the package.json


  if (typeof license === 'string') {
    return { ...data,
      license
    };
  } // License from the package.json


  if ((0, _toolLib.isObject)(license) && typeof license.type === 'string') {
    return { ...data,
      license: license.type
    };
  } // License from copying|license|readme file


  const entries = await (0, _promises.readdir)(packagePath, {
    encoding: 'utf8',
    withFileTypes: true
  });
  const files = (0, _sort.default)(_context = (0, _map.default)(_context2 = (0, _filter.default)(entries).call(entries, entry => {
    var _context3;

    return entry.isFile() && !(0, _startsWith.default)(_context3 = entry.name).call(_context3, '.') && regexpFilename.test(entry.name);
  })).call(_context2, entry => entry.name)).call(_context);

  for (const file of files) {
    const content = await (0, _readFile.default)((0, _path.join)(packagePath, file));

    for (const [aliasLicense, regexp] of (0, _toolLib.objectEntries)(_licenseHeuristics.default)) {
      if (regexp.test(content)) {
        return { ...data,
          license: aliasLicense
        };
      }
    }
  } // License over "npm view"


  const {
    stdout: npmViewLicense
  } = await (0, _exec.default)({
    cmd: 'npm',
    args: ['view', name, 'license'],
    log,
    cwd
  });

  if ((0, _trim.default)(npmViewLicense).call(npmViewLicense) !== '') {
    return { ...data,
      license: (0, _trim.default)(npmViewLicense).call(npmViewLicense)
    };
  }

  return data;
};

var _default = getPackageInfo;
exports.default = _default;