"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _forEach = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/for-each"));

var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/trim"));

var _toolLib = require("@kettil/tool-lib");

var _exec = _interopRequireDefault(require("../../cmd/exec"));

var _existFiles = _interopRequireDefault(require("../../cmd/existFiles"));

const configFiles = ['jest.config.js', 'jest.config.ts'];
const options = ['updateSnapshot', 'runInBand', 'watch'];

const jest = async ({
  cwd,
  ci,
  log
}, props) => {
  const files = await (0, _existFiles.default)(configFiles, cwd);

  if (files.length === 0) {
    throw new Error('Jest config file was not found');
  }

  const cmd = './node_modules/.bin/jest';
  const args = ['--colors', '--config', files[0]];

  if (typeof props.project === 'string') {
    args.push('--selectProjects', props.project);
  } else if ((0, _toolLib.isArray)(props.project)) {
    args.push('--selectProjects', ...props.project);
  }

  (0, _forEach.default)(options).call(options, option => {
    if (props[option] === true) {
      args.push(`--${option}`);
    }
  });

  if (ci && props.coverage) {
    args.push('--ci');
    args.push('--silent');
    args.push('--coverage');
    args.push('--coverageReporters', 'text');
    args.push('--coverageReporters', 'text-summary');
  } else if (ci && !props.coverage) {
    args.push('--ci');
    args.push('--no-coverage');
  } else if (!ci && !props.coverage) {
    args.push('--coverageThreshold', '\'{"global":{"statements":0,"branches":0,"functions":0,"lines":0}}\'');
  }

  log.info('Run test');
  const {
    stdout
  } = await (0, _exec.default)({
    cmd,
    args,
    cwd,
    log,
    withInteraction: props.watch,
    withDirectOutput: ci
  });

  if ((0, _trim.default)(stdout).call(stdout) !== '') {
    log.log('');
    log.log(stdout);
    log.log('');
  }
};

var _default = jest;
exports.default = _default;