"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _trim = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/trim"));

const getInitSettings = props => {
  var _props$github;

  const settings = {
    files: [],
    folders: ['.vscode', 'src', 'src/lib'],
    templates: [['vscode/settings.json', '.vscode/settings.json'], ['editorconfig', '.editorconfig'], ['eslintignore', '.eslintignore'], ['gitignore', '.gitignore'], ['npmignore', '.npmignore'], ['prettierignore', '.prettierignore']],
    libraryProduction: [],
    libraryDevelopment: ['carna'],
    packageBin: {},
    packageInit: {
      private: true,
      version: '0.1.0',
      engines: {
        node: '>= 14',
        npm: '>=6'
      },
      author: 'name <email>'
    },
    packageUpdate: {},
    packageScripts: {
      prepare: '[ "$CI" != "" ] || husky install ./node_modules/carna/configs/husky',
      analyse: 'npx carna analyse',
      deps: 'npx carna deps',
      license: 'npx carna license'
    },
    packagePeerDependencies: [],
    github: {
      name: (_props$github = props.github) === null || _props$github === void 0 ? void 0 : (0, _trim.default)(_props$github).call(_props$github)
    }
  };

  if (typeof settings.github.name === 'string' && settings.github.name === '') {
    throw new Error('The github name is empty');
  } // ######################
  // # Github             #
  // ######################


  if (typeof settings.github.name === 'string') {
    settings.folders.push('.github', '.github/workflows');
    settings.templates.push(['github/dependabot.yml', '.github/dependabot.yml']);
    settings.templates.push(['github/CODEOWNERS', '.github/CODEOWNERS']);
    settings.templates.push(['github/workflows/qa.yml', '.github/workflows/qa.yml']);
  } // ######################
  // # Library            #
  // ######################


  if (props.package) {
    if (typeof settings.github.name === 'string') {
      settings.packageInit.repository = {
        type: 'git',
        url: `https://github.com/${settings.github.name}/<repo>`
      };
      settings.packageInit.bugs = {
        url: `https://github.com/${settings.github.name}/<repo>/issues/new`
      };
      settings.templates.push(['github/workflows/release.yml', '.github/workflows/release.yml']);
    } // Temporary until command release


    settings.templates.push(['releaserc.json', '.releaserc.json']); // libraryDevelopment.push('@kettil/semantic-release-config');

    settings.packageInit.publishConfig = {
      access: 'public'
    };
  } // ######################
  // # CLI                #
  // ######################


  if (props.cli) {
    settings.folders.push('src/bin');
    settings.templates.push(['src/bin/index.ts']);
  } // ######################
  // # Babel/Webpack      #
  // ######################


  settings.templates.push(['babel.config.js']);
  settings.templates.push(['webpack.config.js']);
  settings.libraryDevelopment.push('webpack', 'webpack-cli');

  if (props.package && !props.cli) {
    settings.libraryDevelopment.push('@babel/runtime-corejs3');
    settings.packagePeerDependencies.push('@babel/runtime-corejs3');
  } else {
    settings.libraryProduction.push('@babel/runtime-corejs3');
  } // ######################
  // # Typescript         #
  // ######################


  settings.files.push('src/index.ts', 'src/lib/types.ts');
  settings.templates.push(['typescriptrc.json', 'tsconfig.json']);
  settings.templates.push(['typescriptrc.build.json', 'tsconfig.build.json']);
  settings.libraryDevelopment.push('@types/node', 'typescript');
  settings.packageUpdate.module = 'build/index.js';
  settings.packageUpdate.types = 'build/index.d.ts'; // ######################
  // # Build              #
  // ######################

  settings.packageScripts.build = 'npx carna build';
  settings.packageScripts.prebuild = 'rm -rf ./build'; // ######################
  // # React              #
  // ######################

  /*
  // PROD or DEV (with peerDependencies)
  "react"
  "react-dom"
   if (props.package) {
    settings.packagePeerDependencies.push('@babel/preset-react', '@types/react', '@types/react-dom');
  }
  */
  // ######################
  // # Jest               #
  // ######################

  settings.templates.push(['jest.config.js']);
  settings.files.push('tests/shared/.keep', 'tests/type/.keep');
  settings.folders.push('tests/shared', 'tests/type', 'tests/unit', 'tests/integration', 'tests/e2e');
  settings.templates.push(['dummy.test.ts', 'tests/unit/dummy.test.ts'], ['dummy.test.ts', 'tests/integration/dummy.test.ts'], ['dummy.test.ts', 'tests/e2e/dummy.test.ts']);
  settings.packageScripts.test = 'npx carna test';
  settings.packageScripts['test:unit'] = 'npx carna test -p unit -w';
  settings.packageScripts['test:integration'] = 'npx carna test -p integration -w';
  settings.packageScripts['test:e2e'] = 'npx carna test -p e2e -w';
  return settings;
};

var _default = getInitSettings;
exports.default = _default;