"use strict";

var _Object$defineProperty = require("@babel/runtime-corejs3/core-js-stable/object/define-property");

var _interopRequireDefault = require("@babel/runtime-corejs3/helpers/interopRequireDefault");

_Object$defineProperty(exports, "__esModule", {
  value: true
});

exports.default = void 0;

var _filter = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/filter"));

var _includes = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/includes"));

var _map = _interopRequireDefault(require("@babel/runtime-corejs3/core-js-stable/instance/map"));

var _toolLib = require("@kettil/tool-lib");

var _add = _interopRequireDefault(require("../../actions/git/add"));

var _ls = _interopRequireDefault(require("../../actions/git/ls"));

var _staged = _interopRequireDefault(require("../../actions/git/staged"));

var _eslint = require("../../actions/tools/eslint");

var _prettier = require("../../actions/tools/prettier");

var _existFiles = _interopRequireDefault(require("../../cmd/existFiles"));

var _analyseTask = _interopRequireDefault(require("../analyseTask"));

var _testTask = _interopRequireDefault(require("../testTask"));

const testEslint = new RegExp(`(${_eslint.eslintExtensionAll.replace(/,/g, '|')})$`);
const testPrettier = new RegExp(`(${_prettier.prettierExtensionAll.replace(/,/g, '|')})$`);

const gitCommitTask = async argv => {
  const stagedFiles = await (0, _staged.default)(argv, {});

  if (stagedFiles.length === 0) {
    throw new Error('No stage files found');
  }

  const files = await (0, _existFiles.default)(stagedFiles, argv.cwd);
  const eslintFiles = (0, _filter.default)(files).call(files, file => testEslint.test(file));
  const prettierFiles = (0, _filter.default)(files).call(files, file => testPrettier.test(file));
  const mergedFiles = (0, _toolLib.uniqueArray)([...prettierFiles, ...eslintFiles]);
  const unstagedFiles = await (0, _ls.default)(argv, {
    mode: 'all'
  });
  const intersectFiles = (0, _filter.default)(mergedFiles).call(mergedFiles, file => (0, _includes.default)(unstagedFiles).call(unstagedFiles, file));

  if (intersectFiles.length > 0) {
    throw new Error(`The following files were changed after adding:\n${(0, _map.default)(intersectFiles).call(intersectFiles, file => `       ▸ ${file}\n`).join('')}`);
  }

  await (0, _analyseTask.default)({ ...argv,
    ci: false
  }, {
    eslintFiles,
    prettierFiles
  });
  await (0, _add.default)(argv, {
    files
  });
  await (0, _testTask.default)({ ...argv,
    ci: true
  }, {
    coverage: false
  });
  await (0, _testTask.default)({ ...argv,
    ci: true
  }, {
    coverage: true
  });
};

var _default = gitCommitTask;
exports.default = _default;